"
 COPYRIGHT (c) 1989 by Claus Gittinger / eXept Software AG
 COPYRIGHT (c) 2016 Jan Vrany
 COPYRIGHT (c) 2020 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libcomp' }"

"{ NameSpace: Smalltalk }"

DISection subclass:#DIVariableTable
	instanceVariableNames:'variables_count variables'
	classVariableNames:''
	poolDictionaries:'DISectionTypes'
	category:'System-Compiler-Debug Info'
!

!DIVariableTable class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger / eXept Software AG
 COPYRIGHT (c) 2016 Jan Vrany
 COPYRIGHT (c) 2020 LabWare
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!DIVariableTable class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!DIVariableTable methodsFor:'accessing'!

variables
    ^ variables
!

variablesCount
    ^ variables_count

    "Created: / 15-07-2018 / 11:48:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!DIVariableTable methodsFor:'adding & removing'!

addVariable: aDIVariable
    (variables includes: aDIVariable) ifTrue:[ 
        self error: 'Oops, trying to add the same variable twice?'
    ].
    variables := variables copyWith: aDIVariable.
    variables_count := variables_count + 1

    "Created: / 14-07-2018 / 20:52:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 09-06-2020 / 13:25:54 / Jan Vrany <jan.vrany@labware.com>"
!

addVariableNamed:aString isArgument:aBoolean offset:anOnteger physicalBlock: physical lexicalBlock: lexical
    self addVariable:((DIVariable new)
                physicalBlockSourceOffset: (physical isNil ifTrue:[0] ifFalse:[physical startPosition]);
                lexicalBlockSourceOffset: (lexical isNil ifTrue:[0] ifFalse:[lexical startPosition]);
                frameOffset:anOnteger;
                isArgument:aBoolean;
                name:aString)

    "Created: / 06-02-2019 / 22:30:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 07-02-2019 / 21:57:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!DIVariableTable methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    type := DbgInfoSectionVariableTable.
    variables_count := 0.
    variables := #()

    "/ super initialize.   -- commented since inherited method does nothing

    "Modified: / 15-07-2018 / 12:33:58 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!DIVariableTable methodsFor:'reading & writing'!

decode: aStream
    size := aStream nextUnsignedInt16MSB: UninterpretedBytes isBigEndian.
    type := aStream nextByte.     
    variables_count := aStream nextUnsignedInt16MSB: UninterpretedBytes isBigEndian.
    variables := (1 to: variables_count) collect: [ :i | DIVariable decode: aStream ].

    "Created: / 10-07-2018 / 12:50:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 06-02-2019 / 22:28:59 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

encode: aStream
    self encodePrepare: aStream.
    aStream nextPutByte: type.
    aStream nextPutInt16: variables_count MSB: UninterpretedBytes isBigEndian.        
    variables do:[:block | block encode: aStream ].
    self encodeFinish: aStream.

    "Created: / 15-07-2018 / 10:56:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!DIVariableTable methodsFor:'testing'!

isVariableTableSection
    ^ true

    "Created: / 15-07-2018 / 11:27:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!DIVariableTable class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

